;--------------------------------------------------------------------------
;    driver_FDC.s - generic FDC driver for WD279x (Z80)
;
;    Copyright (c) 2012 by Michal Hucik <http://www.ordoz.com>
;
;    Thies program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    driver_FDC is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with driver_FDC; if not, write to the Free Software
;    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
;--------------------------------------------------------------------------

 
;
; 0.1 - 2012-01-04:
;
; 	- pocatecni driver pro FDC WD279[1/3] bez upravy pro HD
;
; 0.2 - 2012-01-07
;
;	- pridana funkce ReadAddr pro analyzu formatu diskety
;	- pridana volba FDC_DI_ON_ENTIRE_PERIOD 
;	- pridana moznost kompilace pro FDC_HD_PATCH
;	- optimalizace kodu se zamerenim na jeho velikost
;

	.module	fdc_driver


	.area	_CODE


;
; Pri FDC_DI_ON_ENTIRE_PERIOD = 1 se budou volat DI a EI ihned po vstupu do driveru
;
FDC_DI_ON_ENTIRE_PERIOD = 0

;
; Pri FDC_HD_PATCH = 1, se bude kompilovat driver pro HD ready radic, ktery pouziva IM1
FDC_HD_PATCH = 0


;
; FDC_HD_PATCH_USE_PIOZ80 = 1
;
; Driver bude povolovat/zakazovat interrupt od PIO-Z80
;
FDC_HD_PATCH_USE_PIOZ80 = 1


;
; FDC_HD_PATCH_RETURN_TO_IM2 = 1
;
; Driver po ukonceni IM1 operaci nastavi CPU do IM2 rezimu
;
FDC_HD_PATCH_RETURN_TO_IM2 = 1


;
; Ridici port PIO Z80 brana A
;
__PIOZ80_CTRL_A	= 0xfc


;
; Zpusob jak pozadame system o DI
;
.macro SYSTEM_DI_REQUEST
	di
.endm


;
; Zpusob jak pozadame system o EI
;
.macro SYSTEM_EI_REQUEST
	ei
.endm


;
; V rutine __FDC_PreparePIOZ80DensityIntmodeIM1 pocitame s tim, ze nam makro PIOZ80_DI
; pri volbe FDC_HD_PATCH_USE_PIOZ80 = 1 nastavi 0. bit v A = 1 !!!
; Pri zmene makra je proto potreba tuto zavislodt vyresit.
;
.macro PIOZ80_DI
	ld a,#0x03
	out (0xfc),a
.endm

;
;
.macro PIOZ80_EI
	ld a,#0x83
	out (0xfc),a
.endm




; porty WD279x 
__FDC_CMD	= 0xd8	; out 
__FDC_STS	= 0xd8	; in 
__FDC_TRACK	= 0xd9	; in, out 
__FDC_SECTOR	= 0xda	; in, out 
__FDC_DATA	= 0xdb	; in, out 
__FDC_DRIVE	= 0xdc	; out 
__FDC_SIDE	= 0xdd	; out, 0. bit 
__FDC_DENSITY	= 0xde	; out, 0. bit 
__FDC_EINT	= 0xdf	; out, 0. bit 



; definice prikazu typ I.
__FDC_CMD_STEP_IN	= 0xa0
__FDC_CMD_STEP_OUT	= 0x80

__FDC_TYPI_HEAD_SPEED6	= 0x03
__FDC_TYPI_HEAD_SPEED12	= 0x02
__FDC_TYPI_HEAD_SPEED20	= 0x01
__FDC_TYPI_HEAD_SPEED30	= 0x00

__FDC_TYPI_NOVERIFY_TRK	= ( 1 << 2 )



; definice prikazu typ II.
__FDC_CMD_READ_SECTOR	= 0x60
__FDC_CMD_WRITE_SECTOR	= 0x40

__FDC_TYPII_MULTI_SEC	= ( 0 << 4 )
__FDC_TYPII_SINGLE_SEC	= ( 1 << 4 )

__FDC_TYPII_DLY_15MS	= ( 0 << 2 )	; lze pouzit i pro typ III.
__FDC_TYPII_NO_DLY_15MS	= ( 1 << 2 )	; lze pouzit i pro typ III.

__FDC_TYPII_VRF_SIDE	= ( 0 << 1 )
__FDC_TYPII_NO_VRF_SIDE	= ( 1 << 1 )

__FDC_TYPII_DATA_SIG_FB	= ( 1 << 0 )	; 0FBh (DAM)
__FDC_TYPII_DATA_SIG_F8	= ( 0 << 0 )	; 0F8h (vypustena DAM)


; definice prikazu typ III.
__FDC_CMD_READ_ADDR	= 0x3b


; definice prikazu typ IV.
__FDC_CMD_BREAK		= 0x2f


; ovladani mechaniky
__FDC_MOTOR_ON		= ( 1 << 7 )
__FDC_DRIVE_SELECT	= ( 1 << 2 )


;
;
; Konfiguracni parametry
;
;



;
;
; Timing diskovych operaci
; 
__fdc_timing::

__fdc_timing_start_time:
	.dw	500
__fdc_timing_track_time:
	.db	6
__fdc_timing_step_time:
	.db	6
__fdc_timing_motor_predef:
	.db	128


;
;
; Tabulka parametru jednotlivych disku
;
; 0. a 1. bit - velikost sektoru
;		0 - 128 B
;		1 - 256 B
;		2 - 512 B
;		3 - 1024 B
;
; 2. bit - pocet stran media
;		0 - jednostranne
;		1 - oboustranne
;
; 3. bit - invertovana data
;		0 - normalni data
;		1 - invertovana data
;
; 4. bit - hustota zaznamu
;		0 - DD
;		1 - HD
;
;__FDC_Drive:
;	.db	0x0e	; e = oboustranne medium, sector_size = 512, invertovana data
;	.db	0x0e
;	.db	0x0e
;	.db	0x0e



;
;
; Provozni parametry
;



;
; Nenulova hodnota znaci, ze motor prave bezi
;
__FDC_Motor::
	.db	0x00



;
; Prave vybrany disk - musi byt v pameti umisten bezprostredne pred __FDC_Requested_drive !!!
;
__FDC_last_drive:
;__FDC_Selected_drive:
	.db	0x00
;
; 
; Pozadovane parametry READ/WRITE operace
;
;__FDC_Requested:
;
;__FDC_Requested_drive:
;	.db	0x00
;__FDC_Requested_track:
;	.db	0x00
;__FDC_Requested_sector:
;	.db	0x00
;__FDC_Requested_dma:
;	.dw	0x0000




;
; Tady si drzime informaci o posledni nastavene stope
;
__FDC_trackTable:
	.db	0x00
	.db	0x00
	.db	0x00
	.db	0x00





;
;
; Sluzba - nasilne vypnouti motoru
;
;
_fdc_force_motor_off::
	xor a
	ld hl,#__FDC_Motor
	SYSTEM_DI_REQUEST
	ld (hl),a
	SYSTEM_EI_REQUEST
	out (__FDC_DRIVE),a
	ret


;
;
; Sluzba -  IM2 odpocet pro prechod mechaniky do stand-by
;
; Tato rutina musi byt volana pouze uvnitr preruseni!
;
; Poskodi AF a HL !!!
;
_fdc_im2_service::
	.db 0xaf		; XOR A, nebo RET
	ld hl,#__FDC_Motor
	cp (hl)
	ret z			; motor nebezi
	dec (hl)
	ret nz			; jeste nenastal cas ho vypnout
	out (__FDC_DRIVE),a	; motor off
	ret


;
; Ve funkcich read_sector a write_sector blokujeme IM2 sluzbu pro vypinani motoru
;
__FDC_enable_im2_service::
	ld a,#0xaf			; XOR A do _fdc_im2_service
	jr __FDC_change_im2_service
__FDC_dissable_im2_service::
	ld a,0xc9			; RET do _fdc_im2_service
__FDC_change_im2_service:
	ld (#_fdc_im2_service),a
	ret


.if FDC_HD_PATCH

; HD_PATCH:
;
; Zakazeme interrupt od PIOZ80.
; Povolime v FDC interrupt mode.
; Nastavime FDC do DD/HD rezimu.
; Nastavime IM1.

__FDC_PreparePIOZ80DensityIntmodeIM1::

.if FDC_HD_PATCH_USE_PIOZ80
	PIOZ80_DI
.else
	ld a,#0x01
.endif

	; Po predchozim prikazu je 0. bit v A = 1, toho vyuzijeme pro
	; nastaveni FDC_EINT a FDC_DENSITY

	out (__FDC_EINT),a
	
	bit 4,1 (ix)			; __FDC_Drive (params)
	jr nz,1$			; jedna se o HD
	xor a				; jedna se o DD

	1$:
	out (__FDC_DENSITY),a

	im 1

	ret

.endif	;FDC_HD_PATCH


;
; Podle (__FDC_Requested_track) v zavislosti na 2. bitu parametru media
; urcime SIDE.
;
; Vystup:
;		Carry = 1 pokud je potreba nastavit SIDE
;		A     = ( SIDE << 3 ) pro verifikaci strany u prikazu R/W SECTOR
;
__FDC_DI_createSideFromTrack::


.if FDC_DI_ON_ENTIRE_PERIOD
.else
	SYSTEM_DI_REQUEST		; pozadame system o DI
.endif


.if FDC_HD_PATCH
	; HD_PATCH:
	;
	call __FDC_PreparePIOZ80DensityIntmodeIM1
.endif


__FDC_createSideFromTrack::

	xor a				; Carry = 0, overeni strany = 0
	bit 2,1 (ix)			; __FDC_Drive (params)
	ret z				; RET, pokud se jedna o jednostranne medium

	ld a, 2 (ix)			; __FDC_Requested_track
	rla
	rla
	rla
	and #0x08			; v A mame na 3.bitu pozadavek na overeni strany

	scf				; Carry = 1
	ret




;
; Pro zvoleny disk pripravi do HL velikost sektoru. 
;
__FDC_PrepareSectorSize::

	;
	; Do HL spocitame skutecnou velikost sektoru
	;
	ld hl,#0x0080
	ld a,1 (ix)	; __FDC_Drive (params)
	and #0x03
	ret z		; 00: 128 B
	ld l,h
	cp #0x03
	jr nz,2$	; 01: 256 B, 10: 512 B
	inc a		; 11: 1024 B	
	2$:
	ld h,a
	ret





;
;
; Sluzba - Zapis pozadovaneho sektoru
;
; Vstup:
;
;    FDC_DRIVE_ID,                       /* id zvolene mechaniky 0 - 3 */
;    FDC_DRIVE_PARAM,                    /* parametry media */
;    unsigned char log_track,            /* logicka stopa */ 
;    unsigned char phy_sector,           /* fyzicky sektor */
;    void* dma                           /* adresa dat */
;
;
; Vystup:
;		 0: OK
;		-1: ERROR
;		-2: WR_PROTECT
;
_fdc_write_sector::

	ld a,#__FDC_CMD_WRITE_SECTOR|__FDC_TYPII_SINGLE_SEC|__FDC_TYPII_NO_DLY_15MS|__FDC_TYPII_VRF_SIDE|__FDC_TYPII_DATA_SIG_FB|0x10
	ld hl,#__FDC_WriteData
	ld de,#__FDC_WriteData_cpl
	jr __FDC_ReadWrite
 
;
;
; Sluzba - Precteni pozadovaneho sektoru
;
; Vstup:
;
;    FDC_DRIVE_ID,                       /* id zvolene mechaniky 0 - 3 */
;    FDC_DRIVE_PARAM,                    /* parametry media */
;    unsigned char log_track,            /* logicka stopa */ 
;    unsigned char phy_sector,           /* fyzicky sektor */
;    void* dma                           /* adresa dat */
;
;
; Vystup:
;		 0: OK
;		-1: ERROR
;
_fdc_read_sector::

	ld a,#__FDC_CMD_READ_SECTOR|__FDC_TYPII_SINGLE_SEC|__FDC_TYPII_NO_DLY_15MS|__FDC_TYPII_VRF_SIDE|__FDC_TYPII_DATA_SIG_FB|0x10
	ld hl,#__FDC_ReadData
	ld de,#__FDC_ReadData_cpl	

__FDC_ReadWrite::
	push ix
	ld ix,#0x0004			; nastavime index 0 na 1 parametr funkce
	add ix, sp


.if FDC_DI_ON_ENTIRE_PERIOD
	SYSTEM_DI_REQUEST
.else
	ld b,a
	call __FDC_dissable_im2_service
	ld a,b
.endif


	ld (#__FDC_ReadWrite_command),a		; ulozime prikaz operace


	; budeme invertovat data?
	xor a
	bit 3, 1 (ix)	; __FDC_Drive (params) - invertovane medium?
	jr z, 1$	; ne, data nebudou invertovana
	ld a,#0x2f	; cpl
	1$:
	ld (de), a	; ulozime cpl/nop do DRQ rutiny

.if FDC_HD_PATCH
	ld (__FDC_WriteData1_cpl),a		; u HD_PATCH si u zapisu potrebujeme 1 bajt pripravit uz pred DRQ rutinou
.endif


.if FDC_HD_PATCH
	; HD_PATCH:
	;
	; Zkopirujeme pozadovanou DRQ rutinu z HL na adresu 0x0038
	;
	; Pozn: puvodni obsah 0x0038 se nezachova!
	;
	ld de,#0x0038
	ld bc,#0x0008
	ldir
.endif


.if FDC_HD_PATCH
	ld l,4 (ix)				; __FDC_Requested_dma
	ld h,5 (ix)				; do HL
	ld (#__FDC_ReadWrite_dma_addr),hl	; nachystame si ji pred DRQ rutinu
.endif


	call __FDC_PrepareSectorSize		; skutecna velikost do HL


.if FDC_HD_PATCH
	; HD_PATCH:
	;
	; HL - velikost sektoru pripravime pro dalsi pouziti v testu,
	; ktery nasleduje az po ukoncene DRQ rutine
	; (test zda souhlasi pocet prectenych bajtu s velikosti sektoru)

	ld (#__FDC_ReadWrite_sector_size),hl
.else
	; not HD_PATCH:
	; 
	; READ: HL - velikost sektoru upravime do formatu jaky je vyzadovan
	; rutinou __FDC_ReadData
	;
	; WRITE - HL pouzijeme pri testu po ukonceni DRQ rutiny

	ld a,(#__FDC_ReadWrite_command)
	bit 5,a
	jr nz, 2$				; jedna se o READ
	ld (#__FDC_WriteData_sector_size), hl
	jr 4$

	2$:
	ld a,h					; priprava definice velikosti sektoru 
	ld h,l					; pro cteci rutinu
	ld l,a					; prehodime obsah HL: H = L, L = H
	or a					; Pokud se chystame cist mene, nez 256 B,
	jr nz,3$
	inc l					; tak musime nastavit L = 1

	3$:
	ld (#__FDC_ReadData_sector_size), hl	; nastavime velikost sektoru

	4$:
.endif	;FDC_HD_PATCH


	call __FDC_driveON
	ld b,#0x05				; max pocet pokusu

__FDC_ReadWrite_nextAttempt:	
	push bc

	call __FDC_setSectorSideTrack		; nastav stopu, sector a stranu

	jr nc, __FDC_ReadWrite_error		; Nastaveni se nezdarilo?

	; udelame test na WR_PRT
	ld a,(#__FDC_ReadWrite_command)
	bit 5,a
	jr nz, 1$				; jedna se o READ, tedy nas WR_PRT nezajima
	call __FDC_SendBreakWhileIsBusy
	in a, (__FDC_STS)
	bit 6,a
	ld a,#0xfe
	jr z,__FDC_ReadWrite_done		; WR_PRT? return ( -2 )
	call __FDC_delay90us


	1$:

	call __FDC_DI_createSideFromTrack	; Zakazeme preruseni a do A pripravime overeni strany
						; V pripade FDC_HD_PATCH zde
						; zaroven ihned po DI
						; zavola
						; __FDC_PreparePIOZ80DensityIntmodeIM1

	; provedeme XOR prikazu a SIDE 
	.db 0xee				; XOR n
__FDC_ReadWrite_command:
	.db 0x00

	out (__FDC_CMD),a

	call __FDC_waitForNotBusy


.if FDC_HD_PATCH
        ; HD_PATCH:
        ;
	ld c,#__FDC_STS

	.db 0x21				; LD HL,nm
__FDC_ReadWrite_dma_addr:
	.dw 0x0000				; do HL pripravime DMA addr

	; tohle je potreba jen pred zapisem, ale nic se nestane pokud to
	; udelame zbytecne i pred ctenim
	ld a,(hl)				; pred zapisem si pripravime 1 bajt
__FDC_WriteData1_cpl:
	.db 0x00				; cpl/nop

	ei

	1$:
	in b,(c)				; dokud je BUSY, tak 
	rrc b					; cekame v INT na 
	jr nc,1$				; DRQ

	di

	xor a
	out (__FDC_EINT),a
	out (__FDC_DENSITY),a


.if FDC_HD_PATCH_RETURN_TO_IM2
	im 2
.endif


.if FDC_HD_PATCH_USE_PIOZ80
	PIOZ80_EI
.endif


.if FDC_DI_ON_ENTIRE_PERIOD
.else
	SYSTEM_EI_REQUEST
.endif


	; prenesli jsme ocekavany pocet bajtu?
	ex de,hl
	.db 0x01				; LD BC,nm
	__FDC_ReadWrite_sector_size:
	.dw 0x0000

	ld l,4 (ix)				; __FDC_Requested_dma
	ld h,5 (ix)				; do HL

	add hl,bc
	scf
	ccf
	sbc hl,de
	jr nz, __FDC_ReadWrite_error		; pocet zapsanych bajtu nesouhlasi s velikosti sektoru!

.else	;FDC_HD_PATCH

        ; no HD_PATCH:
        ;

	ld a,(#__FDC_ReadWrite_command)
	bit 5,a
	jr nz, __FDC_ReadWrite_read				; jedna se o READ
	
	; operace zapisu
	call __FDC_WriteData			; Dokud je BUSY, tak se pokousime zapisovat data

.if FDC_DI_ON_ENTIRE_PERIOD
.else
	SYSTEM_EI_REQUEST
.endif

	ld a,#0x03
	call __FDC_delay1MSxA

	; zapsali jsme ocekavany pocet bajtu?
	.db 0x01				; LD BC,nm
__FDC_WriteData_sector_size:
	.dw 0x0000

	ld l,4 (ix)				; __FDC_Requested_dma
	ld h,5 (ix)				; do HL

	add hl,bc
	dec de
	scf
	ccf
	sbc hl,de
	jr nz, __FDC_ReadWrite_error		; pocet zapsanych bajtu nesouhlasi s velikosti sektoru!
	jr __FDC_ReadWrite_drq_done



	__FDC_ReadWrite_read:
	; operace cteni
	call __FDC_ReadData			; Cteme data, pokud je vse OK, tak Carry = 1


.if FDC_DI_ON_ENTIRE_PERIOD
.else
	SYSTEM_EI_REQUEST
.endif

	jr nc, __FDC_ReadWrite_error			; Cteni bylo ukonceno s chybou?


	call __FDC_waitForCmdDone

.endif	;FDC_HD_PATCH


__FDC_ReadWrite_drq_done::



	; overime status
	ld a,(#__FDC_ReadWrite_command)
	bit 5,a
	jr nz, 1$				; jedna se o READ

	; zaverecne overeni statusu po zapisu
	in a,(__FDC_STS)
	inc a
	scf					; pokud je vse OK, tak vratime Carry = 1
	jr z, __FDC_ReadWrite_done		; Status bez chyby? return ( 0 )
	jr __FDC_ReadWrite_error

	1$:
	; zaverecne overeni statusu po cteni
	in a,(__FDC_STS)
	cpl
	and #0xbf				; maska podstatnych status bitu po READ_SECTOR
	scf					; pokud je to OK, tak Carry = 1
	jr z, __FDC_ReadWrite_done		; Status bez chyby?


__FDC_ReadWrite_error:
	call __FDC_VerifyTrackByAddr		; overime format diskety a stopu podle ADDR

	pop bc
	djnz __FDC_ReadWrite_nextAttempt		; zkusime to cteni zopakovat 5x


	ld l,#0xff				; return ( -1 )
	jr __FDC_ReadWrite_return

__FDC_ReadWrite_done:
	pop bc
	ld l,a					; return ( A )

__FDC_ReadWrite_return:


.if FDC_DI_ON_ENTIRE_PERIOD
	SYSTEM_EI_REQUEST
.else
	call __FDC_enable_im2_service
.endif

	pop ix
	ret













.if FDC_HD_PATCH
;
; DRQ rutina pro zapis pod IM1
;
__FDC_WriteData::
	out (__FDC_DATA),a	; 2 B
	inc hl			; 1 B
	ld a,(hl)		; 1 B
__FDC_WriteData_cpl:
	.db 0x00		; 1 B - cpl, nebo nop
	ei			; 1 B
	ret			; 2 B 


.else	;FDC_HD_PATCH

;
; Dokud je BUSY, tak cekame na DRQ - kdyz prijde, tak zapiseme pripraveny bajt.
;
; Pokud zmizi BUSY, tak se vratime s Carry = 1 a rotovanym statusem v A
;
__FDC_WriteData::
	ld c,#__FDC_DATA

	ld e,4 (ix)		; __FDC_Requested_dma
	ld d,5 (ix)		; do DE

	ld hl,#__FDC_WriteData_prepare_byte

__FDC_WriteData_prepare_byte:
	ld a,(de)		; precti bajt z DMA
__FDC_WriteData_cpl:
	.db 0x00		; cpl, nebo nop
	inc de
	ld b,a			; do B pripraveny bajt

	1$:
	in a,(__FDC_STS)
	rra
	ret c			; uz neni BUSY, Carry = 1
	rra
	jr c,1$			; cekame na DRQ

	out (c),b		; zapis bajt
	jp (hl)			; jp (hl) ma jen 1 bajt a spotrebuje 4T

.endif	;FDC_HD_PATCH





.if FDC_HD_PATCH
; HD_PATCH
;
; DRQ rutina pro cteni pod IM1
;
__FDC_ReadData::
	in a,(__FDC_DATA)	; 2 B
__FDC_ReadData_cpl:
	.db 0x00		; 1 B - cpl, nebo nop
	ld (hl),a		; 1 B
	inc hl			; 1 B
	ei			; 1 B
	ret			; 2 B


.else	;FDC_HD_PATCH

; not HD_PATCH:
;
; Pokud prijde DRQ, tak precteme pozadovany pocet bajtu.
;
; Vraci:
;		Carry = 1: OK
;		Carry = 0: ERROR - timeout
;
__FDC_ReadData::
	.db 01			; LD BC,nm
__FDC_ReadData_sector_size:
	.dw 0x0002		; sector size
	ld de,#0x0000		; error timer

	ld l,4 (ix)		; __FDC_Requested_dma
	ld h,5 (ix)		; do HL

	; cekame na data
	__FDC_ReadData_dec_timer:
	dec de
	ld a,d
	or e
	ret z			; timeout, Carry = 0

	__FDC_ReadData_read_sts:
	in a,(__FDC_STS)
	and #0x02
	jr nz,__FDC_ReadData_dec_timer		; cekame na DRQ

	; DRQ rutina
	in a,(__FDC_DATA)

__FDC_ReadData_cpl:
	.db 0x00		; nop, nebo cpl

	ld (hl),a

	inc hl
	djnz __FDC_ReadData_read_sts
	dec c
	jp nz,__FDC_ReadData_read_sts

	scf			; OK, Carry = 1
	ret

.endif	;FDC_HD_PATCH



;	
;
;
; Zapnuti pozadovaneho disku.
;
; Vstup:
;		IX
;		drive_id ulozeno v (__FDC_Requested_drive)
;
; Vystup:
;		-
;
__FDC_driveON::
	ld bc,#__FDC_last_drive
	ld a,(bc)

	ld de,#__FDC_trackTable
	ld h,#0x00
	ld l,a
	add hl,de

	in a,(__FDC_TRACK)
	ld (hl),a		; ulozit aktualni stopu disku, ktery byl naposledy vybran

	ld a,0 (ix)		; __FDC_Requested_drive

	ld (bc),a		; novy disk ulozit do __FDC_last_drive

	ld h,#0x00
	ld l,a
	add hl,de		; __FDC_trackTable nove vybraneho disku

	ld a,(hl)		; do A nacist predchozi stopu nove zvolene mechaniky

	out (__FDC_TRACK),a	; a vlozit ji do FDC
	
	ld a,(bc)		; novy drive_id do A

;
; Samotny start motoru, bez inicializace stopy
;
; drive_id je v A
;
;__FDC_init_driveON::
	or #__FDC_MOTOR_ON|__FDC_DRIVE_SELECT

;
; Tohle uz neni potreba - fci blokujeme pres __FDC_dissable_im2_service
;
;.if FDC_DI_ON_ENTIRE_PERIOD
;.else
;	SYSTEM_DI_REQUEST		; pozadame system o DI
;.endif

	out (__FDC_DRIVE),a


	ld a, (#__fdc_timing_motor_predef)
	ld b,a
	ld hl,#__FDC_Motor
	ld a,(hl)
	ld (hl),b

;
; Tohle uz neni potreba - fci blokujeme pres __FDC_dissable_im2_service
;
;.if FDC_DI_ON_ENTIRE_PERIOD
;.else
;	SYSTEM_EI_REQUEST		; pozadame system o EI
;.endif


	and a
	ret nz				; motor uz bezel, nemusime dal na nic cekat 


	ld hl,(#__fdc_timing_start_time)

	; pockame pozadovany pocet ms, nez se nam ustali otacky motoru
	1$:
	djnz 1$
	dec hl
	ld a,h
	or l
	jr nz,1$
	ret




;
; Podle (__FDC_Requested) a podle parametru aktivniho disku (media) nastavime sector, track a
; side.
;
; Vystup:
;		Carry = 1: OK
;		Carry = 0: ERROR
;
__FDC_setSectorSideTrack::
	ld a,3 (ix)			; __FDC_Requested_sector
	cpl
	out (__FDC_SECTOR),a

	call __FDC_createSideFromTrack	; v Carry obdrzime zda je potreba nastavit SIDE

	ld a,2 (ix)			; __FDC_Requested_track
	jr c,1$				; je potreba nastavit stranu?
	add a,a				; inverujeme 0. bit

	1$:
	out (__FDC_SIDE),a

	srl a
	ld h,a				; pozadovana stopa do H

	in a,(__FDC_TRACK)
	cpl
	sub h
	ccf				; Neguj Carry.
					; Pokud je v FDC pozadovana stopa,
					; tak Carry = 1.
					; V pripade, ze jsou odlisne, tak
					; Carry urcuje kterym smerem bude
					; potreba krokovat.

	ret z				; v FDC je jiz nastavena pozadovana stopa

	; Pokud je v FDC nastavena jina stopa, tak budeme 
	; krokovat pozadovanym smerem.

	ld l,a				; v L je rozdil mezi pozadovanou a
					; aktualni stopou

	jr nc,step_in$

	step_out$:

	call __FDC_StepOut
	dec l
	jr nz,step_out$
	jr step_done$

	step_in$:

	call __FDC_StepIn
	inc l
	jr nz,step_in$

	; krokovani dokonceno
	step_done$:
	in a,(__FDC_TRACK)		; overime zda uz je v FDC nastavena
	xor h				; pozadovana stopa
	add a,#1			; Pokud ano, Carry = 1.
					; Pri chybe Carry = 0.

;
; Pockame 1 ms * (__fdc_timing_track_time)
;
__FDC_delayByTrackSettingTime::
	ld a,(#__fdc_timing_track_time)
;
; Pockame 1 ms * A
;
__FDC_delay1MSxA::
	push bc
	ld b,#0x00
	1$:
	djnz 1$
	dec a
	jr nz,1$
	pop bc
	ret



;
; Overeni stopy
;
__FDC_VerifyTrackByAddr::
	ld a,#0xff


;
; Tohle uz neni potreba - fci blokujeme pres __FDC_dissable_im2_service
;
;.if FDC_DI_ON_ENTIRE_PERIOD
;.else
;	SYSTEM_DI_REQUEST		; pozadame system o DI
;.endif

	ld (#__FDC_Motor),a		; citac motoru na maximalni hodnotu

;
; Tohle uz neni potreba - fci blokujeme pres __FDC_dissable_im2_service
;
;.if FDC_DI_ON_ENTIRE_PERIOD
;.else
;	SYSTEM_EI_REQUEST		; pozadame system o DI
;.endif

	ld h,#0x74			; timeout timer

	1$:
	in a,(__FDC_STS)		; az nebude BUSY
	rra
	jr c,2$				; tak proved overeni stopy

	dec hl
	ld a,h
	or l
	jr nz,1$

	jr __FDC_SendBreakWhileIsBusy	; timeout, nedockali jsme se NOT
					; BUSY, takze jdeme posilat BREAK, dokud to neozije


	; overeni stopy pomoci READ_ADDR
	2$:
	in a,(__FDC_TRACK)
	rra
	call c,__FDC_StepIn
	call nc,__FDC_StepOut

	call __FDC_delayByTrackSettingTime

.if FDC_DI_ON_ENTIRE_PERIOD
.else
	SYSTEM_DI_REQUEST
.endif

	ld a,#__FDC_CMD_READ_ADDR | __FDC_TYPII_NO_DLY_15MS
	out (__FDC_CMD),a

	call __FDC_VerifyTrackByAddr_read_data

.if FDC_DI_ON_ENTIRE_PERIOD
.else
	SYSTEM_EI_REQUEST
.endif

	jr nc, __FDC_StepToTRK00	; READ_ADDR skoncil s chybou,
					; takze se pokusime udelat restore
	call __FDC_waitForCmdDone

	; zaverecna kontrola statusu
	in a,(__FDC_STS)
	cpl
	and #0xbf
	jr nz, __FDC_StepToTRK00	; spatny status? krokuj do TRK00
	
	ld a,c
	out (__FDC_TRACK),a		; nastavime stopu podle ADDR

	ret


;
; Precteni dat z ADDR
;
; Vraci:
;		Carry = 1: OK
;		Carry = 0: ERROR - timeout
;		reg. C   : prectena stopa
;
__FDC_VerifyTrackByAddr_read_data::
	ld l,#2				; stst maska
	ld d,#0x0f			; timeout timer

	1$:
	dec de
	ld a,d
	or e
	ret z				; timeout, Carry = 0

	in a,(__FDC_STS)
	and l
	jr nz,1$			; cekame na DRQ

	in a,(__FDC_DATA)
	ld c,a				; precteny track do C

	ld b,#5				; ted precteme a zahodime zbyvajicich 5 bajtu
	
	2$:
	in a,(__FDC_STS)
	and l
	jr nz, 2$
	
	in a,(__FDC_DATA)
	djnz 2$

	scf				; Carry = 1
	ret



;
; Budeme krokovat dolu, dokud nedojdeme na nultou stopu - Restore TRK00
;
;
__FDC_StepToTRK00::
	ld l,#0x50			; pocitame s tim, ze neni vic jak 80 stop

	1$:
	call __FDC_StepOut
	dec l
	jr z, __FDC_delayByTrackSettingTime
	in a,(__FDC_STS)
	bit 2,a
	jr nz, 1$			; opakuj dokud neni status TRK00

	ld a,#0xff
	out (__FDC_TRACK),a		; track = 0

	jr __FDC_delayByTrackSettingTime



;
; StepIN - krok o stopu nahoru
;
__FDC_StepIn::
	ld a,#__FDC_CMD_STEP_IN | __FDC_TYPI_HEAD_SPEED6 | __FDC_TYPI_NOVERIFY_TRK
	jr __FDC_StepInOut
;
; StepOUT - krok o stopu dolu
;
__FDC_StepOut::
	ld a,#__FDC_CMD_STEP_OUT | __FDC_TYPI_HEAD_SPEED6 | __FDC_TYPI_NOVERIFY_TRK

__FDC_StepInOut:
	out (__FDC_CMD),a

	ld a,(#__fdc_timing_step_time)
	call __FDC_delay1MSxA



;
; Dokud bude BUSY, tak budeme opakovane posilat __FDC_CMD_BREAK
;
__FDC_SendBreakWhileIsBusy::
	ld a,#__FDC_CMD_BREAK
	out (__FDC_CMD),a


;
; Cekame dokud je BUSY
;
__FDC_waitForCmdDone::
	call __FDC_delay90us

	1$:
	in a,(__FDC_STS)
	rra
	jr c,__FDC_delay90us		; pokud uz neni BUSY, tak wait a potom RET
	djnz 1$
	jr __FDC_SendBreakWhileIsBusy	; pokud je stale BUSY, tak posilej BREAK az do soudneho dne



;
; Pockame, dokud nezmizi priznak BUSY, nebo dokud B > 0
;
__FDC_waitForNotBusy::
	call __FDC_delay90us
	1$:
	in a,(__FDC_STS)
	and #0x81
	xor #0x80
	ret z		; not BUSY
	djnz 1$
	ret		; timeout


;
; Pockame cca 90 us
;
__FDC_delay90us::
	ld b,#0x18
	1$:
	djnz 1$
	ret




